% Initialize
clear all; close all; clc;

instrreset;     % Disconnect and delete all instrument objects

try
    % Instrument connection string
    deviceAddr = 'TCPIP0::10.0.1.31';                   % For LAN
    % deviceAddr = 'GPIB0::22'';                        % For GPIB
    % deviceAddr = 'USB0::0x0957::0x1A07::MY53000101';  % For USB
    
    % Connect to instrument
    myDmm = visa('agilent',deviceAddr);                 % Create VISA object
%     get(myDmm)                                        % Return current settings for all parameters
    set(myDmm, 'InputBufferSize', 400000);              % Set large buffer size to hold output from DMM
    set(myDmm, 'Timeout', 15);                          % Set timeout to 15 sec
    fopen(myDmm);                                       % Open connection
    
    % Query instrument
    instrument_ID = query(myDmm, '*IDN?');
    disp(['Instrument: ' instrument_ID]);

    % Configure instrument from reset condition in series of steps
    fprintf(myDmm, '*RST');
    
    % 1. Configure acquisition
    fprintf(myDmm,'CONFIGURE:VOLTAGE:DC');                      % Select DCV measurements
    fprintf(myDmm,'SENSE:VOLTAGE:DC:RANGE 10');                 % Select 10V range
    fprintf(myDmm,'SENSE:VOLTAGE:DC:IMPEDANCE:AUTO OFF');       % 10M input impedance setting , 'ON' sets >10Gohm
    fprintf(myDmm,'SENSE:VOLTAGE:DC:ZERO:AUTO OFF');            % Auto-zero OFF
    fprintf(myDmm,'SENSE:VOLTAGE:DC:NULL:STATE OFF');           % Null function OFF
    fprintf(myDmm,'SENSE:VOLTAGE:DC:NPLC 0.02');                % Integration time = 0.02 PLC
    CheckDMMError(myDmm);                                       % Check DMM for errors
    
    % 2. Configure trigger
    fprintf(myDmm,'TRIGGER:SOURCE IMMEDIATE');                  % Trigger source: IMMEDIATE, EXTERNAL, BUS
    fprintf(myDmm,'TRIGGER:SLOPE POSITIVE');                    % Trigger slope: POSITIVE, NEGATIVE
    fprintf(myDmm,'TRIGGER:COUNT 1');                           % Trigger count (Number of triggers): 1 to 1000000, or INF
    fprintf(myDmm,'TRIGGER:DELAY:AUTO OFF');                    % Automatic trigger delay: OFF
    fprintf(myDmm,'TRIGGER:DELAY 0');                           % Trigger delay: 0 sec (yields ~1100 samples/sec)
    CheckDMMError(myDmm);                                       % Check DMM for errors
    
    % 3. Configure sampling
    fprintf(myDmm,'SAMPLE:COUNT 100000');                       % Sample count (Number of samples/trigger): 1 to 1000000
    CheckDMMError(myDmm);                                       % Check DMM for errors
    
    % 4. Turn off statistics, calculations, display, etc.
    fprintf(myDmm,'CALCULATE:SCALE:STATE OFF');                 % Auto scale: OFF
    fprintf(myDmm,'CALCULATE:AVERAGE:STATE OFF');               % Averaging: OFF
    fprintf(myDmm,'CALCULATE:LIMIT:STATE OFF');                 % Limit testing: OFF
    fprintf(myDmm,'CALCULATE:TRANSFORM:HISTOGRAM:STATE OFF');   % Histogram: OFF
    fprintf(myDmm,'DISPLAY:STATE OFF');                         % Display: OFF (has no effect on data transfer to PC)
    CheckDMMError(myDmm);                                       % Check DMM for errors    
    
    % 5. Initiate measurement    
    %    - Total number of measurements = TRIGGER:COUNT * SAMPLE:COUNT
    %    - Put DMM in "wait for trigger" state
    %    - DMM will trigger immediately with 'TRIGGER:SOURCE IMMEDIATE'
    fprintf(myDmm,'INITIATE');                                  
    
    % Acquire data
    data = zeros(100000,1);                                     % Array for storing numeric data
    
    %  - Continuously read measurements from the instrument
    startTime = tic;
    for k = 1:2000
        dataString = query(myDmm, 'DATA:REMOVE? 50, WAIT');                 % Get 50 measurements
        data(1+(k-1)*50 : k*50) = str2double(strsplit(dataString, ',')).';  % Convert data to double format
    end
    elapsedTime = toc(startTime);    
    fprintf(myDmm,'DISPLAY:STATE ON');                          % Display: ON
    
    % Summary of data acquisition    
    disp(['Data remaining  : ' num2str(str2num(strtrim(query(myDmm, 'DATA:POINTS?'))))]);
    disp(['Time elapsed    : ' num2str(elapsedTime) ' sec']);
    disp(['Acquisition rate: ' num2str(100000/elapsedTime) ' samples/sec']);
        
    % Plot
    plot(1:100000, data);
    
    % Clean up
    fclose(myDmm);
    delete(myDmm);
    clear myDmm;
    
catch MExc
    MExc.message
    instrreset;     % Disconnect and delete all instrument objects
end
